@@ -8,11 +8,12 @@ import CommonTypes
 import Patterns
 import UU.Pretty(text,PP_Doc,empty,(>-<))
 import TokenDef
-import List (intersperse)
-import Char
+import Data.List (intersperse)
+import Data.Char
 import Scanner (Input(..),scanLit,input)
-import List
+import Data.List
 import Expression
+import Macro --marcos
 import UU.Scanner.Token
 import UU.Scanner.TokenParser
 import UU.Scanner.GenToken
@@ -33,47 +34,57 @@ pIdentifier   = uncurry Ident <$> pVaridPos
 
 
 parseAG :: Options -> [FilePath] -> String -> IO (AG,[Message Token Pos])
-parseAG opts searchPath file 
-              = do (es,_,_,mesg) <- parseFile opts searchPath file
+parseAG opts searchPath file
+              = do (es,_,_,_,mesg) <- parseFile False opts searchPath file
                    return (AG es, mesg)
 
+--marcos
+parseAGI :: Options -> [FilePath] -> String -> IO (AG, Maybe String)
+parseAGI opts searchPath file
+              = do (es,_,_,ext,_) <- parseFile True opts searchPath file
+                   return (AG es, ext)
+
+
 depsAG :: Options -> [FilePath] -> String -> IO ([String], [Message Token Pos])
 depsAG opts searchPath file
-  = do (_,_,fs,mesgs) <- parseFile opts searchPath file
+  = do (_,_,fs,_,mesgs) <- parseFile False opts searchPath file
        return (fs, mesgs)
 
-parseFile :: Options -> [FilePath] -> String -> IO  ([Elem],[String],[String],[Message Token Pos ])
-parseFile opts searchPath file 
+-- marcos: added the parameter 'agi' and the 'ext' part
+parseFile :: Bool -> Options -> [FilePath] -> String -> IO  ([Elem],[String],[String], Maybe String,[Message Token Pos ])
+parseFile agi opts searchPath file
  = do txt <- readFile file
       let litMode = ".lag" `isSuffixOf` file
           (files,text) = if litMode then scanLit txt
                          else ([],txt)
           tokens       = input opts (initPos file) text
 
-          steps = parse pElemsFiles tokens
-          stop (_,fs,_,_) = null fs
-          cont (es,fs,allfs,msg)
+          steps = parse (pElemsFiles agi) tokens
+          stop (_,fs,_,_,_) = null fs
+          cont (es,fs,allfs,ext,msg)
             = do files <- mapM (resolveFile searchPath) fs
-                 res <- mapM (parseFile opts searchPath) files
-                 let (ess,fss,allfss, msgs) = unzip4 res
-                 return (es ++ concat ess, concat fss, concat allfss ++ allfs, msg ++ concat msgs)
-      let (Pair (es,fls) _ ,mesg) = evalStepsMessages steps
+                 res <- mapM (parseFile agi opts searchPath) files
+                 let (ess,fss,allfss,_, msgs) = unzip5 res
+                 return (es ++ concat ess, concat fss, concat allfss ++ allfs, ext, msg ++ concat msgs)
+      let (Pair (es,fls,ext) _ ,mesg) = evalStepsMessages steps
       let allfs = files ++ fls
-      loopp stop cont (es,allfs,allfs,mesg)
+      loopp stop cont (es,allfs,allfs, ext,mesg)
  where
-  
+
     --
     -- Option dependent AG Parsers inlined here
     -- to have access to the opts
     -- while retaining sharing
     --
-    
-    pElemsFiles :: AGParser ([Elem],[String])
-    pElemsFiles = pFoldr (($),([],[])) pElem'
+
+    pElemsFiles :: Bool -> AGParser ([Elem],[String],Maybe String)
+    pElemsFiles agi = pFoldr (($),([],[],Nothing)) pElem'
        where pElem' =  addElem <$> pElem
                    <|> pINCLUDE *> (addInc <$> pStringPos)
-             addElem e      (es,fs) = (e:es,   fs)
-             addInc  (fn,_) (es,fs) = (  es,fn:fs)
+                   <|> pEXTENDS *> (addExt <$> pStringPos)
+             addElem e      (es,fs,ext) = (e:es,   fs, ext)
+             addInc  (fn,_) (es,fs,ext) = (  es,fn:fs, ext)
+             addExt  (fn,_) (es,fs,ext) = if agi then (es,fs, Just fn) else (es,fn:fs, ext) --marcos
 
     pCodescrapL = (\(ValToken _ str pos) -> (str, pos))<$>
                         parseScrapL <?> "a code block"
@@ -93,14 +104,16 @@ parseFile opts searchPath file
     pElems :: AGParser Elems
     pElems = pList_ng pElem
 
-    pComplexType =  List   <$> pBracks pTypeEncapsulated 
+    pComplexType =  List   <$> pBracks pTypeEncapsulated
                <|> Maybe  <$ pMAYBE <*> pType
                <|> Either <$ pEITHER <*> pType <*> pType
                <|> Map    <$ pMAP <*> pTypePrimitive <*> pType
                <|> IntMap <$ pINTMAP <*> pType
+               <|> OrdSet <$ pSET <*> pType
+               <|> IntSet <$ pINTSET
                <|> tuple  <$> pParens (pListSep pComma field)
       where field = (,) <$> ((Just <$> pIdentifier <* pTypeColon) `opt` Nothing) <*> pTypeEncapsulated
-            tuple xs = Tuple [(fromMaybe (Ident ("x"++show n) noPos) f, t) 
+            tuple xs = Tuple [(fromMaybe (Ident ("x"++show n) noPos) f, t)
                              | (n,(f,t)) <- zip [1..] xs
                              ]
 
@@ -112,10 +125,11 @@ parseFile opts searchPath file
                  <*> pList pIdentifier
                  <*> pOptAttrs
                  <*> pAlts
-                <*> pSucceed False
+                 <*> pSucceed False
         <|> Attr <$> pATTR
                  <*> pOptClassContext
                  <*> pNontSet
+                 <*> pOptQuantifiers
                  <*> pAttrs
         <|> Type <$> pTYPE
                  <*> pOptClassContext
@@ -127,6 +141,7 @@ parseFile opts searchPath file
                  <*> pOptClassContext
                  <*> pNontSet
                  <*> pOptAttrs
+                 <*> pOptQuantifiers
                  <*> pSemAlts
         <|> Set  <$> pSET
                  <*> pIdentifierU
@@ -153,9 +168,15 @@ parseFile opts searchPath file
                  <*> pCodescrap'
                  <*> pCodescrap'
                  <*> pCodescrap'
-        <|> codeBlock <$> (pIdentifier <|> pSucceed (Ident "" noPos)) <*> ((Just <$ pATTACH <*> pIdentifierU) <|> pSucceed Nothing) <*> pCodeBlock <?> "a statement"
-              where codeBlock nm mbNt (txt,pos) = Txt pos nm mbNt (lines txt)
+        <|> codeBlock <$> pBlockKind <*> ((Just <$ pATTACH <*> pIdentifierU) <|> pSucceed Nothing) <*> pCodeBlock <?> "a statement"
+              where codeBlock knd mbNt (txt,pos) = Txt pos knd mbNt (lines txt)
 
+    pBlockKind :: AGParser BlockKind
+    pBlockKind =
+          BlockPragma <$ pOPTPRAGMAS
+      <|> BlockImport <$ pIMPORTS
+      <|> BlockMain   <$ pTOPLEVEL   -- block is moved to the toplevel ("main") module when "sepsemmods" is used
+      <|> pSucceed BlockOther
 
     pAttrs :: AGParser Attrs
     pAttrs
@@ -173,15 +194,15 @@ parseFile opts searchPath file
 
     pSingleInhAttrDef :: AGParser (Identifier,Type,(String,String,String))
     pSingleInhAttrDef
-      = (\v tp -> (v,tp,("","",""))) <$> pIdentifier <* pTypeColon <*> pType <?> "inh attribute declaration"
+      = (\v tp -> (v,tp,("","",""))) <$> pAttrIdentifier <* pTypeColon <*> pTypeOrSelf <?> "inh attribute declaration"
 
     pSingleSynAttrDef :: AGParser (Identifier,Type,(String,String,String))
     pSingleSynAttrDef
-      = (\v u tp -> (v,tp,u)) <$> pIdentifier <*> pUse <* pTypeColon <*> pType <?> "syn attribute declaration"
-      
+      = (\v u tp -> (v,tp,u)) <$> pAttrIdentifier <*> pUse <* pTypeColon <*> pTypeOrSelf <?> "syn attribute declaration"
+
     pSingleChnAttrDef :: AGParser (Identifier,Type,(String,String,String))
     pSingleChnAttrDef
-      = (\v tp -> (v,tp,("","",""))) <$> pIdentifier <* pTypeColon <*> pType <?> "chn attribute declaration"
+      = (\v tp -> (v,tp,("","",""))) <$> pAttrIdentifier <* pTypeColon <*> pTypeOrSelf <?> "chn attribute declaration"
 
     pOptAttrs :: AGParser Attrs
     pOptAttrs = pAttrs `opt` Attrs noPos [] [] []
@@ -189,15 +210,15 @@ parseFile opts searchPath file
     pInhAttrNames :: AGParser AttrNames
     pInhAttrNames
                    = (\vs tp -> map (\v -> (v,tp,("","",""))) vs)
-                      <$> pIdentifiers <*  pTypeColon <*> pType <?> "attribute declarations"
+                      <$> pAttrIdentifiers <*  pTypeColon <*> pTypeOrSelf <?> "attribute declarations"
 
     pAttrNames :: AGParser AttrNames
     pAttrNames
              = (\vs use tp -> map (\v -> (v,tp,use)) vs)
-                 <$> pIdentifiers <*> pUse <* pTypeColon <*> pType <?> "attribute declarations"
+                 <$> pAttrIdentifiers <*> pUse <* pTypeColon <*> pTypeOrSelf <?> "attribute declarations"
 
     pAlt :: AGParser Alt
-    pAlt =  Alt <$> pBar <*> pSimpleConstructorSet <*> pFields <?> "a datatype alternative"
+    pAlt =   (Alt <$> pBar <*> pSimpleConstructorSet <*> pList_ng pIdentifier <*> pFields <*> pMaybeMacro <?> "a datatype alternative")
 
     pAlts :: AGParser Alts
     pAlts =  pList_ng pAlt <?> "datatype alternatives"
@@ -207,10 +228,10 @@ parseFile opts searchPath file
 
     pField :: AGParser Fields
     pField
-           =  (\nms tp -> map (flip (,) tp) nms)
-              <$> pIdentifiers <* pTypeColon <*> pType
-              <|> (\s -> [(Ident (mklower (getName s)) (getPos s) ,NT s [])]) <$> pIdentifierU
-
+           =   (\nms tp -> map (\nm -> FChild nm tp) nms) <$> pAttrIdentifiers <* pTypeColon <*> pType
+           <|> (\s   -> [FChild (Ident (mklower $ getName s) (getPos s)) (NT s [] False)]) <$> pIdentifierU
+           <|> (\t   -> [FCtx [t]]) <$ pSmallerEqual <*> pTypePrimitive
+    
     pSemAlt :: AGParser SemAlt
     pSemAlt = SemAlt <$> pBar <*> pConstructorSet <*> pSemDefs <?> "SEM alternative"
 
@@ -223,11 +244,11 @@ parseFile opts searchPath file
           <|>                            pLOC              *> pList1 pLocDecl
           <|>                            pINST             *> pList1 pInstDecl
           <|>  pSEMPRAGMA *> pList1 (SemPragma <$> pNames)
-          <|> (\n e -> [AugmentDef n e]) <$ pAugmentToken <*> pIdentifier <* pAssign <*> pExpr
-          <|> (\n e -> [AroundDef n e]) <$ pAROUND <*> pIdentifier <* pAssign <*> pExpr
+          <|> (\n e -> [AugmentDef n e]) <$ pAugmentToken <*> pAttrIdentifier <* pAssign <*> pExpr
+          <|> (\n e -> [AroundDef n e]) <$ pAROUND <*> pAttrIdentifier <* pAssign <*> pExpr
           <|> (\a b -> [AttrOrderBefore a [b]]) <$> pList1 pAttrOrIdent <* pSmaller <*> pAttrOrIdent
           <|> (\sources target nt expr -> [MergeDef target nt sources expr]) <$ pMERGE <*> (pList1_ng pIdentifier <* pAS <|> pSucceed []) <*> pIdentifier <* pTypeColon <*> pIdentifierU <* pAssign <*> pExpr
-          <|> (\mbNm pat (owrt,pos) exp -> [Def pos mbNm (pat ()) exp owrt]) <$> pMaybeRuleName <*> pPattern (const <$> pAttr) <*> pAssignPos <*> pExpr
+          <|> (\mbNm pat (owrt,pos,pur,eager) exp -> [Def pos mbNm (pat ()) exp owrt pur eager]) <$> pMaybeRuleName <*> pPattern (const <$> pAttr) <*> pRuleSym <*> pExpr
 
     pMaybeRuleName :: AGParser (Maybe Identifier)
     pMaybeRuleName
@@ -236,10 +257,10 @@ parseFile opts searchPath file
 
     pAttrDef :: AGParser (Maybe Identifier -> Identifier -> SemDef)
     pAttrDef
-      = (\pat (owrt,pos) exp mbNm fld -> Def pos mbNm (pat fld) exp owrt)
-               <$ pDot <*> pattern <*> pAssignPos <*> pExpr
+      = (\pat (owrt,pos,pur,eager) exp mbNm fld -> Def pos mbNm (pat fld) exp owrt pur eager)
+               <$ pDot <*> pattern <*> pRuleSym <*> pExpr
       where pattern =  pPattern pVar
-                   <|> (\ir a fld -> ir $ Alias fld a (Underscore noPos) []) <$> ((Irrefutable <$ pTilde) `opt` id) <*> pIdentifier
+                   <|> (\ir a fld -> ir $ Alias fld a (Underscore noPos)) <$> ((Irrefutable <$ pTilde) `opt` id) <*> pAttrIdentifier
 
     pLocDecl :: AGParser SemDef
     pLocDecl = pDot <**> (pIdentifier <**> (pTypeColon <**> (   (\(tp,pos) _ ident _  -> TypeDef pos ident tp) <$> pLocType
@@ -262,18 +283,36 @@ parseFile opts searchPath file
         then pDoubleColon
         else pColon
 
+    --marcos
+    pMaybeMacro :: AGParser MaybeMacro
+    pMaybeMacro  =  Just <$ pDoubleArrow <*>  pMacro
+                <|> pSucceed Nothing
+
+    pMacro :: AGParser Macro
+    pMacro
+          =  Macro <$> pIdentifierU
+                   <*> pList1 pMacroChild
+                   <?> "macro"
+
+    pMacroChild :: AGParser MacroChild
+    pMacroChild
+          =  (pIdentifier <* pEquals) <**>
+             (flip RuleChild   <$> pMacro       <|>
+              flip ChildChild  <$> pIdentifier  <|>
+              flip ValueChild  <$> pCodescrap' )
+
     --
     -- End of AG Parser
     --
 
 resolveFile :: [FilePath] -> FilePath -> IO FilePath
-resolveFile path fname = search (path ++ ["."])                                                  
+resolveFile path fname = search (path ++ ["."])
  where search (p:ps) = do let filename = joinPath [p, fname]
                           fExists <- doesFileExist filename
                           if fExists
                             then return filename
                             else search ps
-       search []     = error ("File: " ++ show fname ++ " not found in search path: " ++ show (concat (intersperse ";" (path ++ ["."]))) )                      
+       search []     = error ("File: " ++ show fname ++ " not found in search path: " ++ show (concat (intersperse ";" (path ++ ["."]))) )
 
 pathSeparator = "/"
 
@@ -309,7 +348,7 @@ pNontSet = set0
   where set0 = pChainr (Intersect <$ pIntersect) set1
         set1 = pChainl (Difference <$ pMinus) set2
         set2 = pChainr (pSucceed Union) set3
-        set3 = pIdentifierU <**> opt (flip Path <$ pArrow <*> pIdentifierU) NamedSet 
+        set3 = pIdentifierU <**> opt (flip Path <$ pArrow <*> pIdentifierU) NamedSet
             <|> All <$ pStar
             <|> pParens set0
 
@@ -332,10 +371,13 @@ pClassContext :: AGParser ClassContext
 pClassContext
   = pListSep pComma ((,) <$> pIdentifierU <*> pList pTypeHaskellAnyAsString)
 
+pOptQuantifiers :: AGParser [String]
+pOptQuantifiers = (return <$ pDoubleColon <*> pCodescrap') `opt` []
+
 pTypeNt :: AGParser Type
 pTypeNt
-  =   ((\nt -> NT nt []) <$> pIdentifierU <?> "nonterminal name (no brackets)")
-  <|> (pParens (NT <$> pIdentifierU <*> pList pTypeHaskellAnyAsString) <?> "nonterminal name with parameters (using parenthesis)")
+  =   ((\nt -> mkNtType nt []) <$> pIdentifierU <?> "nonterminal name (no brackets)")
+  <|> (pParens (mkNtType <$> pIdentifierU <*> pList pTypeHaskellAnyAsString) <?> "nonterminal name with parameters (using parenthesis)")
 
 pTypeHaskellAnyAsString :: AGParser String
 pTypeHaskellAnyAsString
@@ -347,7 +389,7 @@ pTypeHaskellAnyAsString
 pTypeEncapsulated :: AGParser Type
 pTypeEncapsulated
   =   pParens pTypeEncapsulated
-  <|> NT <$> pIdentifierU <*> pList pTypeHaskellAnyAsString
+  <|> mkNtType <$> pIdentifierU <*> pList pTypeHaskellAnyAsString
   <|> (Haskell . getName) <$> pIdentifier
   <|> pTypePrimitive
 
@@ -359,11 +401,22 @@ pType :: AGParser Type
 pType =  pTypeNt
      <|> pTypePrimitive
 
+pTypeOrSelf :: AGParser Type
+pTypeOrSelf = pType <|> Self <$ pSELF
+
 pIdentifiers :: AGParser [Identifier]
 pIdentifiers =  pList1Sep pComma pIdentifier <?> "lowercase identifiers"
 
+pAttrIdentifier :: AGParser Identifier
+pAttrIdentifier = pIdentifier
+                  <|> (\pos -> Ident "imports" pos) <$> pIMPORTS
+                  <|> (\pos -> Ident "toplevel" pos) <$> pTOPLEVEL
+
+pAttrIdentifiers :: AGParser [Identifier]
+pAttrIdentifiers = pList1Sep pComma pAttrIdentifier <?> "lowercase identifiers"
+
 pUse :: AGParser (String,String,String)
-pUse = (  (\u x y->(x,y,show u)) <$> pUSE <*> pCodescrap'  <*> pCodescrap')` opt` ("","","") <?> "USE declaration"
+pUse = (  (\u x y->(x,y,show u)) <$> pUSE <*> pCodescrap'  <*> pCodescrap') `opt` ("","","") <?> "USE declaration"
 
 mklower :: String -> String
 mklower (x:xs) = toLower x : xs
@@ -389,10 +442,10 @@ pFieldIdentifier =  pIdentifier
 pAugmentToken :: AGParser ()
 pAugmentToken = () <$ (pAUGMENT <|> pPlus)
 
-pAttr = (,) <$> pFieldIdentifier <* pDot <*> pIdentifier
+pAttr = (,) <$> pFieldIdentifier <* pDot <*> pAttrIdentifier
 
 pAttrOrIdent
-  = OccAttr <$> pFieldIdentifier <* pDot <*> pIdentifier
+  = OccAttr <$> pFieldIdentifier <* pDot <*> pAttrIdentifier
   <|> OccRule <$> pIdentifier
 
 nl2sp :: Char -> Char
@@ -405,15 +458,17 @@ pLocType = (\u -> (Haskell $ getName u, getPos u)) <$> pIdentifierU
        <|> (\(s,p) -> (Haskell s,p)) <$> pCodescrap  <?> "a type"
 
 pVar :: AGParser (Identifier -> (Identifier, Identifier))
-pVar = (\att fld -> (fld,att)) <$> pIdentifier
+pVar = (\att fld -> (fld,att)) <$> pAttrIdentifier
 
 pAssign :: AGParser Bool
 pAssign =  False <$ pReserved "="
        <|> True  <$ pReserved ":="
 
-pAssignPos :: AGParser (Bool, Pos)
-pAssignPos  =   (\p -> (False,p)) <$> pReserved "="
-            <|> (\p -> (True,p))  <$> pReserved ":="
+pRuleSym :: AGParser (Bool, Pos, Bool, Bool)
+pRuleSym  =     (\p -> (False, p, True, False)) <$> pReserved "="
+            <|> (\p -> (True,  p, True, False)) <$> pReserved ":="
+            <|> (\p -> (False, p, True, False)) <$> pReserved "<-"
+            <|> (\p -> (False, p, True, True))  <$> pReserved "<<-"
 
 pPattern :: AGParser (a -> (Identifier,Identifier)) -> AGParser (a -> Pattern)
 pPattern pvar = pPattern2 where
@@ -422,8 +477,8 @@ pPattern pvar = pPattern2 where
                <|> pPattern1 <?> "a pattern"
   pPattern1 =  pvariable
            <|> pPattern2
-  pvariable = (\ir var pat a -> case var a of (fld,att) -> ir $ Alias fld att (pat a) []) 
-           <$> ((Irrefutable <$ pTilde) `opt` id) <*> pvar <*> ((pAt *> pPattern1) `opt` const (Underscore noPos)) 
+  pvariable = (\ir var pat a -> case var a of (fld,att) -> ir $ Alias fld att (pat a))
+           <$> ((Irrefutable <$ pTilde) `opt` id) <*> pvar <*> ((pAt *> pPattern1) `opt` const (Underscore noPos))
   pPattern2 = (mkTuple <$> pOParenPos <*> pListSep pComma pPattern0 <* pCParen )
           <|> (const . Underscore) <$> pUScore <?> "a pattern"
     where mkTuple _ [x] a = x a
@@ -438,9 +493,11 @@ pCodescrap ::  AGParser (String,Pos)
 pCodescrap   = pCodeBlock
 
 pSEM, pATTR, pDATA, pUSE, pLOC,pINCLUDE, pTYPE, pEquals, pColonEquals, pTilde,
+      pEXTENDS, --marcos
       pBar, pColon, pLHS,pINST,pSET,pDERIVING,pMinus,pIntersect,pDoubleArrow,pArrow,
       pDot, pUScore, pEXT,pAt,pStar, pSmaller, pWRAPPER, pNOCATAS, pPRAGMA, pMAYBE, pEITHER, pMAP, pINTMAP,
-      pMODULE, pATTACH, pUNIQUEREF, pINH, pSYN, pAUGMENT, pPlus, pAROUND, pSEMPRAGMA, pMERGE, pAS
+      pMODULE, pATTACH, pUNIQUEREF, pINH, pSYN, pAUGMENT, pPlus, pAROUND, pSEMPRAGMA, pMERGE, pAS, pSELF,
+      pIMPORTS, pOPTPRAGMAS, pSmallerEqual, pINTSET
       :: AGParser Pos
 pSET         = pCostReserved 90 "SET"     <?> "SET"
 pDERIVING    = pCostReserved 90 "DERIVING"<?> "DERIVING"
@@ -454,6 +511,7 @@ pEXT         = pCostReserved 90 "EXT"     <?> "EXT"
 pATTR        = pCostReserved 90 "ATTR"    <?> "ATTR"
 pSEM         = pCostReserved 90 "SEM"     <?> "SEM"
 pINCLUDE     = pCostReserved 90 "INCLUDE" <?> "INCLUDE"
+pEXTENDS     = pCostReserved 90 "EXTENDS" <?> "EXTENDS" --marcos
 pTYPE        = pCostReserved 90 "TYPE"    <?> "TYPE"
 pINH         = pCostReserved 90 "INH"     <?> "INH"
 pSYN         = pCostReserved 90 "SYN"     <?> "SYN"
@@ -462,6 +520,7 @@ pMAYBE       = pCostReserved 5  "MAYBE"   <?> "MAYBE"
 pEITHER      = pCostReserved 5  "EITHER"  <?> "EITHER"
 pMAP         = pCostReserved 5  "MAP"     <?> "MAP"
 pINTMAP      = pCostReserved 5  "INTMAP"  <?> "INTMAP"
+pINTSET      = pCostReserved 5  "INTSET"  <?> "INTSET"
 pUSE         = pCostReserved 5  "USE"     <?> "USE"
 pLOC         = pCostReserved 5  "loc"     <?> "loc"
 pLHS         = pCostReserved 5  "lhs"     <?> "loc"
@@ -479,6 +538,7 @@ pBar         = pCostReserved 5  "|"       <?> "|"
 pIntersect   = pCostReserved 5  "/\\"     <?> "/\\"
 pMinus       = pCostReserved 5  "-"       <?> "-"
 pDoubleArrow = pCostReserved 5  "=>"      <?> "=>"
+pSmallerEqual= pCostReserved 5  "<="      <?> "<="
 pArrow       = pCostReserved 5  "->"      <?> "->"
 pStar        = pCostReserved 5  "*"       <?> "*"
 pSmaller     = pCostReserved 5  "<"       <?> "<"
@@ -488,3 +548,7 @@ pAUGMENT     = pCostReserved 5  "AUGMENT" <?> "AUGMENT"
 pAROUND      = pCostReserved 5  "AROUND" <?> "AROUND"
 pMERGE       = pCostReserved 5  "MERGE" <?> "MERGE"
 pAS          = pCostReserved 5  "AS" <?> "AS"
+pSELF        = pCostReserved 5  "SELF" <?> "SELF"
+pIMPORTS     = pCostReserved 5  "imports" <?> "imports"
+pOPTPRAGMAS  = pCostReserved 5  "optpragmas" <?> "optpragmas"
+pTOPLEVEL    = pCostReserved 5  "toplevel" <?> "toplevel"
