@@ -38,16 +38,17 @@ data NTAttr = NTAInh NontermIdent Identifier Type -- nt, attribute, type
 getNtaNameType (NTAInh nt name tp) = (name,tp)
 getNtaNameType (NTASyn nt name tp) = (name,tp)
 
-getAttr     (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = name
-getIsIn     (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = ii
-getHasCode  (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = hc
-getLhsNt    (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = nt
-getCon      (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = con
-getField    (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = field
-getRhsNt    (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = childnt
-getType     (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = tp
-getDefines  (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = defines
-getUses     (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses) = uses
+getAttr     (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = name
+getIsIn     (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = ii
+getHasCode  (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = hc
+getLhsNt    (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = nt
+getCon      (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = con
+getField    (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = field
+getRhsNt    (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = childnt
+getType     (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = tp
+getDefines  (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = defines
+getUses     (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = uses
+getExplicit (CRule name ii hc nt con field childnt tp pattern rhs defines owrt origin uses expl _) = expl
 
 isLocal = (_LOC==) . getField
 isInst = (_INST==) . getField
@@ -76,14 +77,14 @@ ntattr cr  | isLocal cr =  Nothing
                            in Just (at (getNt cr) (getAttr cr) (fromJust (getType cr)))
 
 cRuleLhsInh :: Identifier -> NontermIdent -> ConstructorIdent -> Type -> CRule
-cRuleLhsInh attr nt con tp = CRule attr True False nt con _LHS Nothing (Just tp) (error "cRuleLhsInh") [] Map.empty False "" Set.empty
+cRuleLhsInh attr nt con tp = CRule attr True False nt con _LHS Nothing (Just tp) (error "cRuleLhsInh") [] Map.empty False "" Set.empty False Nothing
 cRuleTerminal :: Identifier -> NontermIdent -> ConstructorIdent -> Type -> CRule
-cRuleTerminal attr nt con tp = CRule attr True False nt con _LOC Nothing (Just tp) (error ("cRuleTerminal: " ++ show (attr, nt, con, tp))) [] Map.empty False "" Set.empty
+cRuleTerminal attr nt con tp = CRule attr True False nt con _LOC Nothing (Just tp) (error ("cRuleTerminal: " ++ show (attr, nt, con, tp))) [] Map.empty False "" Set.empty False Nothing
 cRuleRhsSyn :: Identifier -> NontermIdent -> ConstructorIdent -> Type -> Identifier -> NontermIdent -> CRule
-cRuleRhsSyn attr nt con tp field childnt = CRule attr True False nt con field (Just childnt) (Just tp) (error ("cRuleRhsSyn: " ++ show (attr, nt, con, tp, field))) [] Map.empty False "" Set.empty
+cRuleRhsSyn attr nt con tp field childnt = CRule attr True False nt con field (Just childnt) (Just tp) (error ("cRuleRhsSyn: " ++ show (attr, nt, con, tp, field))) [] Map.empty False "" Set.empty False Nothing
 
 defaultRule :: Identifier -> NontermIdent -> ConstructorIdent -> Identifier -> CRule
-defaultRule attr nt con field =  CRule attr (er 1) (er 2) nt con field (er 3) (er 4) (er 5) (er 6) (er 7) (er 8) (er 9) (er 10)
+defaultRule attr nt con field =  CRule attr (er 1) (er 2) nt con field (er 3) (er 4) (er 5) (er 6) (er 7) (er 8) (er 9) (er 10) False Nothing
                                  where er i = error ("Default rule has no code " ++ show i)
 
 instance Eq CRule where
