@@ -1,31 +1,47 @@
 {-# LANGUAGE MagicHash,
-             UnboxedTuples #-}
+             UnboxedTuples, FlexibleInstances #-}
 
 module TokenDef where
 
+
+
 import GHC.Prim
 import UU.Scanner.Token
+
 import UU.Scanner.GenToken
+
 import UU.Scanner.GenTokenOrd
+
 import UU.Scanner.Position
+
 import UU.Parsing.MachineInterface(Symbol(..))
-import Char(isPrint,ord)
+import Data.Char(isPrint,ord)
 import HsToken
 import CommonTypes
 
 
 
+
 instance Symbol Token  where
+
  deleteCost (Reserved key _) = case key of
                 "DATA"         -> 7#
                 "EXT"          -> 7#
+
                 "ATTR"         -> 7#
+
                 "SEM"          -> 7#
+
                 "USE"          -> 7#
+
                 "INCLUDE"      -> 7#
+
                 _              -> 5#
+
  deleteCost (ValToken v _  _) = case v of
+
                 TkError -> 0#
+
                 _       -> 5#
 
 
